package ca.uwaterloo.ece651.fahrenheitconverter;

import android.test.ActivityInstrumentationTestCase2;
import android.test.UiThreadTest;
import android.widget.EditText;

import org.junit.Test;

import java.math.BigDecimal;

public class MainActivityTest extends ActivityInstrumentationTestCase2<MainActivity> {

    private MainActivity mainActivity ;
    private EditText fahrenheitInput ;
    private EditText celsiusOutput ;

    /* Constructor of the test class */
    public MainActivityTest() {
        super(MainActivity.class);
    }


    /* Sets up the test case, initializing variable declarations for
       the Activity that we want to test.

       This method executes prior to each @Test method.
     */
    @Override
    protected void setUp() throws Exception {
        super.setUp();

        mainActivity = getActivity();

        fahrenheitInput = (EditText) mainActivity.findViewById(R.id.fahrenheitInput) ;
        celsiusOutput = (EditText) mainActivity.findViewById(R.id.celsiusOutput) ;
    }

    @Test
    public void testPreConditions() {

        assertNotNull(mainActivity) ;
        assertNotNull(fahrenheitInput) ;
        assertNotNull(celsiusOutput) ;

        assertTrue(fahrenheitInput.getText().toString().isEmpty()) ;
        assertTrue(celsiusOutput.getText().toString().isEmpty()) ;
    }

    /* To test conversion, we set the input text field directly, and assert if
       the value in celsiusOutput is what we expect it to be.
       Note that setting fahrenheitInput requires us to be inside the UI thread;
       thus, we use the @UiThreadTest annotation.
     */

    @UiThreadTest
    public void testConversion() {
        assertEquals(convert(32), 0.0) ;
        assertEquals(convert(15), -9.44) ;
        assertEquals(convert(16), -8.89) ;
        assertEquals(convert(100), 37.78) ;
        assertEquals(convert(-22), -30.0) ;
    }

    private double convert(Integer fahrenheit) {
        return convert(fahrenheit * 1.0) ;
    }

    private double convert(Double fahrenheit) {
        fahrenheitInput.setText(fahrenheit.toString());
        return new BigDecimal(
                new Double(celsiusOutput.getText().toString())
        ).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue() ;
    }
}